close all;
clear;
%% 
Num_real=30; % number of realisations, 30 is suggested

Fs=200; % sampling frequency

t=0:1/Fs:10-(1/Fs);

f1=5;
f2=11;
f3=18;
part_length=floor(length(t)/6);

k1=1;
k2=1;
k3=1;
k4=1;

% Synchrony at f1 (5 Hz) and f3 (18 Hz) during 0-1.67 s and 3.33-5 s respectively
x=[k1*cos(2*pi*f1*t(1:part_length)) k2*cos(2*pi*f2*t(1:part_length)) k3*cos(2*pi*f3*t(1:part_length))];
y=[k1*cos(2*pi*f1*t(1:part_length)) k4*cos(2*pi*f3*t(1:part_length)) k4*cos(2*pi*f3*t(1:part_length))];
x(1000)=x(end);
y(1000)=y(end);
x=awgn(x,10,'measured'); % corruped by WGN, 10 dB SNR
y=awgn(y,10,'measured'); % corruped by WGN, 10 dB SNR

t=[];
t=(1:length(x))/Fs;

%% Plot  spectrogram

w=3*Fs/5;
w_ov=round(w*0.9);
nfft=w;
[s_x,f,t_spect]=spectrogram(x,w,w_ov,nfft,Fs,'yaxis');
s_x_amplitude=abs(s_x);

[s_y,f,t_spect]=spectrogram(y,w,w_ov,nfft,Fs,'yaxis');
s_y_amplitude=abs(s_y);

figure;
subplot(2,1,1);
surf(t_spect,f,s_x_amplitude);
title('x');
axis tight;
view(0,90); shading interp;
ylim([0 30]);
% colorbar;
colormap jet;
xlabel('Time (s)');
ylabel('Frequency (Hz)');

subplot(2,1,2);
surf(t_spect,f,s_y_amplitude);
title('y');
axis tight;
view(0,90); shading interp;
ylim([0 30]);
% colorbar;
colormap jet;
xlabel('Time (s)');
ylabel('Frequency (Hz)');
%% Perform NA-MEMD
No_noise_channels=10; % 10 adjacent WGN channels
Noise_power=mean([var(x) var(y)]); % power of the WGN channels is equal to the average power of signals x and y

for real=1:Num_real

    real
 
    noise_process=zeros(length(x),No_noise_channels);
  
    for i=1:No_noise_channels
        noise_process(:,i)=randn(length(x),1).*sqrt(Noise_power); % generate 10 channels of WGN process
    end
    
    memd_input=[x' y' noise_process]; % concatenate the input signals x and y with the additional WGN channels

    IMF=memd(memd_input);

%     %% Plot IMFs of obtained from only the first realisation
%     if real==1
%         column=3;
%         row=size(IMF,2);
% 
%         figure('name','IMFs');
%         subplot(row+1,column,1);plot(t,memd_input(:,1),'k');
%         title('x');
%         ylabel('Magnitude');
%         axis tight;
%          ylim([-1 1]);
% 
%         subplot(row+1,column,2);plot(t,memd_input(:,2),'k');
%         title('y');
%         ylabel('Magnitude');
%         axis tight;
%         ylim([-2 2]);
% 
%         subplot(row+1,column,3);plot(t,memd_input(:,3),'k');
%         title('WGN');
%         ylabel('Magnitude');
%         axis tight;
%         ylim([-1 1]);
% 
%         for i=1:column
%             for j=1:row
% 
%                 subplot(row+1,column,((j)*column)+i);
% 
%                 if i==1
%                     plot(t,squeeze(IMF(1,j,:)),'k');
%                 elseif i==2
%                     plot(t,squeeze(IMF(2,j,:)),'k');   
%                 elseif i==3
%                     plot(t,squeeze(IMF(3,j,:)),'k');
%                 end
% 
%                 ylabel(num2str(j));
% 
%                 if j==size(IMF,2)
%                     xlabel('Time (s)');
%                 end
% 
%                 axis tight;
%                 ylim([-1 1]);
% 
%             end
%         end
%     end
    %% Combine (sum) three IMFs with maximum powers

    variance_x=[];
    for i=1:size(IMF,2);
        variance_x(i)=var(IMF(1,i,:));
    end
    [temp,max_var_x]=sort(variance_x,'descend');
    
    temp2=sum(IMF(1:2,max_var_x(1:3),:),2);
    
    %% Set parameters used by ISC
 
    fr=0.05; % frequency resolution in Hz
    alpha=5; % alpha value used for creating gaussian window in stft function
    W_STFT=Fs; % window length
    
    [Cxy_CWT,f,t,pairs]=ISC_CWT(temp2,Fs,fr); % Perform CWT-based ISC
    [Cxy_STFT,f,t,pairs]=ISC_STFT(temp2,Fs,fr,5,W_STFT,alpha); % Perform STFT-based ISC

    % Convert from 'double' data format to 'single' data format to save memory space
    Cxy_CWT_all(real,:,:)=single(squeeze(Cxy_CWT(1,1,:,:)));
    Cxy_CWT=[];
    
    Cxy_STFT_all(real,:,:)=single(squeeze(Cxy_STFT(1,1,:,:)));
    Cxy_STFT=[];
    
end
%% Average ISC values over 'Num_real' realisations

Cxy_CWT_ave=squeeze(mean(Cxy_CWT_all));
Cxy_STFT_ave=squeeze(mean(Cxy_STFT_all));
%% Plot CWT-based ISC in logarithmic scale

figure;
imagesc(t,f,log10(Cxy_CWT_ave));
axis tight;
 set(gca,'YDir','normal');
ylim([0 30]);
colormap jet;
xlabel('Time (s)');
ylabel('Frequency (Hz)');
xlim([W_STFT/2 length(t)-W_STFT/2]/Fs);
title('CWT-based ISC');
%% Plot STFT-based ISC in logarithmic scale

figure;
imagesc(t,f,log10(Cxy_STFT_ave));
axis tight;
 set(gca,'YDir','normal');
ylim([0 30]);
colormap jet;
xlabel('Time (s)');
ylabel('Frequency (Hz)');
xlim([W_STFT/2 length(t)-W_STFT/2]/Fs);
title('STFT-based ISC');
